/*
 Copyright (C) 2010 Christian Dywan <christian@twotoasts.de>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 See the file COPYING for the full license text.
*/

public class Postler.CellRendererToggle : Gtk.CellRendererToggle {
    public string? stock_id { get; set; }
    public string? prelight_stock_id { get; set; }

    public CellRendererToggle () {
    }
    public override void get_size (Gtk.Widget widget, Gdk.Rectangle? cell_area,
        out int x_offset, out int y_offset, out int width, out int height) {

        int icon_width, icon_height;
        int xpad, ypad;
        Gtk.icon_size_lookup_for_settings (widget.get_settings (),
            Gtk.IconSize.MENU, out icon_width, out icon_height);
        Gtk.cell_renderer_get_padding (this, out xpad, out ypad);
        icon_width += xpad * 2;
        icon_height += ypad * 2;

        if (cell_area != null) {
            if (&x_offset != null) {
                x_offset = (int)((widget.get_direction () == Gtk.TextDirection.RTL
                    ? (1.0 - xalign) : xalign) * (cell_area.width - icon_width));
                x_offset = int.max (x_offset, 0);
            }
            if (&y_offset != null) {
                y_offset = (int)(yalign * (cell_area.height - icon_height));
                y_offset = int.max (y_offset, 0);
            }
        }
        else {
            if (&x_offset != null)
                x_offset = 0;
            if (&y_offset != null)
                y_offset = 0;
        }

        if (&width != null)
            width = icon_width;
        if (&height != null)
            height = icon_height;
    }

#if HAVE_GTK3
    public override void render (Cairo.Context context, Gtk.Widget widget,
        Gdk.Rectangle expose_area, Gdk.Rectangle cell_area,
        Gtk.CellRendererState flags) {
        Gdk.Rectangle background_area = expose_area;
#else
    public override void render (Gdk.Window window, Gtk.Widget widget,
        Gdk.Rectangle background_area, Gdk.Rectangle cell_area,
        Gdk.Rectangle expose_area, Gtk.CellRendererState flags) {
        var context = Gdk.cairo_create (window);
#endif

        Gdk.cairo_rectangle (context, expose_area);
        Gdk.cairo_rectangle (context, background_area);

        Gdk.Pixbuf? icon = null;
        /* if (expose_area.intersect (cell_area, null) */
        if (Gdk.rectangle_intersect (expose_area, cell_area, null)
         && (flags & Gtk.CellRendererState.PRELIT) != 0
         && prelight_stock_id != null) {
            icon = widget.render_icon (prelight_stock_id, Gtk.IconSize.MENU, null);
        }
        else if (stock_id != null) {
            icon = widget.render_icon (stock_id, Gtk.IconSize.MENU, null);
        }
        if (icon != null) {
            Gdk.Rectangle rect;
            int xpad, ypad;
            get_size (widget, cell_area,
                out rect.x, out rect.y, out rect.width, out rect.height);
            Gtk.cell_renderer_get_padding (this, out xpad, out ypad);
            rect = new Gdk.Rectangle ();
            rect.x += cell_area.x + xpad;
            rect.y += cell_area.y + ypad;
            rect.width -= xpad * 2;
            rect.height -= ypad * 2;
            Gdk.cairo_set_source_pixbuf (context, icon,
                rect.x, rect.y);
            context.fill ();
        }
    }
}

