/*
 Copyright (C) 2011 Christian Dywan <christian@twotoasts.de>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 See the file COPYING for the full license text.
*/

namespace Postler {
    public class Contact {
        public string display_name;
        public GLib.File? avatar;
        static Regex contact_regex = null;

        public Contact (string display_name, GLib.File? avatar) {
            this.display_name = display_name;
            this.avatar = avatar;
        }

        internal static string[] parse (string address)
            ensures (result[0] != null && result[1] != null) {
            if (address.length < 1) {
                GLib.critical ("parse: assertion '!address.length < 1' failed");
                return { address, address };
            }
            if(contact_regex == null) {
                /* taken from http://www.regular-expressions.info/email.html ; added possibility of "mailto:" */
                var valid_email_address_regex = """(mailto:)?(?<EMAIL>[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4})""";
                var possibly_invalid_email_address_regex = """(mailto:)?(?<EMAIL>.+)""";
                try {
                    contact_regex = new Regex("""^\s*((("(?<NAME>.*)"|'(?<NAME>.*)'|(?<NAME>.*)))?\s*<%s>|%s)\s*$"""
                                              .printf (valid_email_address_regex, possibly_invalid_email_address_regex),
                                              GLib.RegexCompileFlags.CASELESS|GLib.RegexCompileFlags.DUPNAMES);
                }
                catch (GLib.RegexError e) {
                    GLib.critical ("parse: could not parse regular expression");
                    return { address, address };
                }
            }
            MatchInfo mi;
            if(contact_regex.match (address, 0, out mi)) {
                var email = mi.fetch_named ("EMAIL");
                if (email == null || email == "") email = address;
                var name = mi.fetch_named ("NAME");
                if (name == null || name == "") name = email;
                name = name.replace ("\\\"", "`");
                return { name , email };
            }
            else {
                GLib.warning ("parse: could not parse address %s", address);
                return { address, address };
            }
        }

        public static string address_from_string (string contact) {
            return parse (contact)[1];
        }

        public static string name_from_string (string contact) {
            return parse (contact)[0];
        }

        public static bool equal (string contact, string? other)
            requires (contact.chr (-1, '<') == null) {
            if (other == null)
                return false;
            if (other.chr (-1, '<') == null && contact == other)
                return true;
            if (contact == address_from_string (other))
                return true;
            return false;
        }
        
        public static string[] quoted_split (string? comma_separated_list) {
            var retval = new string[0];
            if (comma_separated_list != null) {
                var cur = "";
                foreach (var s in comma_separated_list.split (",")) {
                    if (cur == "") {
                        cur = s;
                    }
                    else {
                        cur += "," + s;
                    }
                    bool quot_open = false;
                    for (var tail = cur.chr (-1, '"'); tail != null; tail = tail[1:tail.length].chr (-1, '"')) {
                        quot_open = !quot_open;
                    }
                    if (!quot_open && cur != "") {
                        retval += cur;
                        cur = "";
                    }
                }
                if (cur != "") {
                    retval += cur;
                }
            }
            return retval;
        }        

    }
}

