/*
 Copyright (C) 2011 Christian Dywan <christian@twotoasts.de>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 See the file COPYING for the full license text.
*/

public class Postler.Folders : Gtk.Toolbar {
    Accounts accounts;

    public Postler.Messages messages { get; set; }
    public string? selected_label { get; private set; }
    public string? selected_location { get; private set; }
    public FolderType selected_type { get; private set; }

    public Folders (Accounts accounts) {
#if HAVE_GTK3
        get_style_context ().add_class ("secondary-toolbar");
#endif

        this.accounts = accounts;
        accounts.add_info.connect ((info) => {
            if (info.type == AccountType.SEARCH) {
                var button = get_nth_item (0) as Gtk.RadioToolButton;
                unowned GLib.SList<Gtk.RadioToolButton>? group = button.get_group ();
                var search = folder_button (group, info.name,
                                            info.path, FolderType.GENERIC);
                insert (search, get_n_items () - 1);
            }
        });

        populate ();
    }

    async void populate () {
        unowned GLib.SList<Gtk.RadioToolButton>? group = null;
        for (int type = 0; type < FolderType.GENERIC; type++) {
            var folder = (FolderType)type;
            var button = folder_button (group, folder.get_label (),
                "file://%/" + folder.get_role () + "/%", folder);
            insert (button, -1);
            group = button.get_group ();
        }

        var separator = new Gtk.SeparatorToolItem ();
        separator.set_expand (true);
        separator.draw = false;
        insert (separator, -1);

        /* foreach (unowned AccountInfo account_info in infos) { */
        for (unowned GLib.List<AccountInfo> info_iter = accounts.get_infos ();
            info_iter != null; info_iter = info_iter.next) {
            AccountInfo account_info = info_iter.data;
            if (account_info.type == AccountType.SEARCH) {
                var search = folder_button (group, account_info.name,
                    account_info.path, FolderType.GENERIC);
                insert (search, -1);
                group = search.get_group ();
            }
        }

        var hide_read = folder_button (group, _("Unread Mail"), "search:unread/1", FolderType.GENERIC);
        insert (hide_read, -1);
    }

    Gtk.RadioToolButton folder_button (GLib.SList<Gtk.RadioToolButton>? group,
        string label, string uri, FolderType folder_type) {

        var button = new Gtk.RadioToolButton (group);
        button.set_data ("uri", uri);
        button.label = label;
        button.is_important = true;
        button.visible = true;
        button.clicked.connect (() => {
            selected_label = label;
            selected_type = folder_type;
            selected_location = uri;
            messages.populate (uri, folder_type);
        });
        return button;
    }

    public void select_folder (string folder) {
        foreach (var child in get_children ()) {
            var button = child as Gtk.RadioToolButton;
            if (button.label == folder
             || button.get_data<string> ("uri") == folder) {
                if (!button.active)
                    button.active = true;
                else
                    button.clicked ();
                break;
            }
        }
    }

    public static string decode_foldername (string name) {
        /* Modified UTF-7 according to RFC 2060 */
        string decoded = name.replace ("&", "+").replace (",", "/");
        try {
            return GLib.convert (decoded, -1, "UTF-8", "UTF-7", null);
        }
        catch (Error error) {
            if (error is ConvertError.ILLEGAL_SEQUENCE)
                GLib.warning ("Incomplete folder name conversion: %s", name);
            else
                GLib.warning ("Failed to convert folder name: %s", error.message);
        }
        return name;
    }

    public void empty_folder (string location) {
        try {
            string[] folders = { "cur", "new" };
            foreach (var folder in folders) {
                var folder_dir = File.new_for_path (location + "/" + folder);
                var folder_enumerator = folder_dir.enumerate_children (
                    FILE_ATTRIBUTE_STANDARD_NAME, 0, null);
                GLib.FileInfo info;
                while ((info = folder_enumerator.next_file (null)) != null) {
                    var file = folder_dir.resolve_relative_path (info.get_name ());
                    file.delete (null);
                }
            }
            messages.clear ();
        }
        catch (GLib.Error error) {
            GLib.critical (_("Failed to empty folder \"%s\": %s"),
                location, error.message);
        }
    }
}
