/*
 Copyright (C) 2010 Christian Dywan <christian@twotoasts.de>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 See the file COPYING for the full license text.
*/

[CCode (cprefix = "GLib", lower_case_cprefix = "g_")]
namespace GLib {
    [CCode (cname = "g_ascii_strdown", cheader_filename = "glib.h")]
    public static string ascii_strdown (string str, ssize_t len = -1);
    [CCode (cname = "g_base64_encode", cheader_filename = "glib.h")]
        public static string base64_encode (string data, size_t length);
    }
    [CCode (cname = "g_memory_input_stream_new_from_data", cheader_filename = "gio/gio.h")]
    public static GLib.MemoryInputStream memory_input_stream_from_data (
        string data, ssize_t len, GLib.DestroyNotify? destroy);

[CCode (cprefix = "Gdk", lower_case_cprefix = "gdk_")]
namespace Gdk {
    [CCode (cname = "gdk_pixbuf_save_to_buffer", cheader_filename = "gdk-pixbuf/gdk-pixdata.h")]
        public static bool pixbuf_save_to_buffer (Gdk.Pixbuf pixbuf, out string buffer,
                                                  out size_t buffer_size, string type,
                                                  void* a, void* b) throws GLib.Error;
    [CCode (cname = "gdk_rectangle_intersect", cheader_filename = "gdk/gdk.h")]
    public static bool rectangle_intersect (Gdk.Rectangle src1, Gdk.Rectangle src2,
                                            out Gdk.Rectangle dest);
    }

[CCode (cprefix = "Gtk", lower_case_cprefix = "gtk_")]
namespace Gtk {
    [CCode (cname = "gtk_cell_renderer_get_padding", cheader_filename = "gtk/gtk.h")]
    public static void cell_renderer_get_padding (Gtk.CellRenderer renderer,
                                                  out int xpad, out int ypad);
    [CCode (cname = "gtk_cell_renderer_get_alignment", cheader_filename = "gtk/gtk.h")]
    public static void cell_renderer_get_alignment (Gtk.CellRenderer renderer,
                                                    out float xalign, out int yalign);
}

[CCode (cprefix = "Folks", lower_case_cprefix = "folks_", cheader_filename = "folks/folks.h")]
namespace Folks {
    public class Individual : GLib.Object, AliasDetails, AvatarDetails, EmailDetails, ImDetails, NameDetails {
        public bool is_user { get; }
        public GLib.List<Folks.Persona> personas { get; set; }
    }
    public class Persona : GLib.Object {
        public string display_id { get; construct; }
    }
    public interface AliasDetails : GLib.Object {
        public abstract string? alias { get; set; }
    }
    public interface AvatarDetails : GLib.Object {
        public abstract GLib.File? avatar { get; set; }
    }
    public interface EmailDetails : GLib.Object {
        public abstract GLib.List<string?> email_addresses { get; set; }
    }
    public interface ImDetails : GLib.Object {
        /* public abstract GLib.HashTable<string, string> im_addresses { get; set; } */
    }
    public interface NameDetails : GLib.Object {
        public abstract string? full_name { get; set; }
    }
    public class IndividualAggregator : GLib.Object {
        public IndividualAggregator ();
        /* public signal void individuals_changed (Gee.Set<Individual> added,
            Gee.Set<Individual> removed, string? message, Persona? actor,
            GroupDetails.ChangeReason reason); */
        public async void prepare () throws GLib.Error;
        public bool is_prepared { get; }
        public Folks.Individual? user { get; }
        public unowned GLib.HashTable<string, Folks.Individual> get_individuals ();
    }
}

